#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "device.h"
#include "callbacks.h"


extern GladeXML * xml;  /* this probably shouldn't be global */

void
show_error_dialog(GtkWindow * parent, const gchar * error)
{
	GtkMessageDialog * errorDialog;

	errorDialog = gtk_message_dialog_new(
			GTK_WINDOW(parent),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE,
			error);

	gtk_dialog_run(GTK_DIALOG(errorDialog));
	gtk_widget_destroy(GTK_WIDGET(errorDialog));
}

void
on_openDeviceButton_toggled            (GtkToggleButton *toggleButton,
                                        gpointer         user_data)
{
	GtkListStore * packetListStore;
	GtkMessageDialog * errorDialog;
	static gint clearButtonTag;
	static gint transmitHandlerId;

	packetListStore = (GtkListStore *)user_data;

	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggleButton)))
	{
		const gchar * deviceFileName;
		gchar * error;

		deviceFileName = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "deviceFileNameEntry")));

		if(!device_init(deviceFileName, packetListStore, (GIOFunc)incoming_packet_handler, &error))
		{
			show_error_dialog(GTK_WINDOW(glade_xml_get_widget(xml, "mainWindow")), error);

			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggleButton), FALSE);
			return;
		}

		/* gray serial port out */
		gtk_widget_set_sensitive(
				glade_xml_get_widget(xml, "deviceHbox"),
				FALSE);

		/* activate transmit button (signal handler registered below) */
		gtk_widget_set_sensitive(
				glade_xml_get_widget(xml, "transmitButton"),
				TRUE);

		gtk_button_set_label(GTK_BUTTON(toggleButton), "Close Device");



		/* register handlers that need iface, ss, and packetListStore */


		transmitHandlerId = g_signal_connect((gpointer) glade_xml_get_widget(xml, "transmitButton"),
						     "clicked",
						     G_CALLBACK(on_transmitButton_clicked),
						     NULL);
	}
	else
	{
		/* reenable device entry box */
		gtk_widget_set_sensitive(
				glade_xml_get_widget(xml, "deviceHbox"),
				TRUE);

		/* deactivate transmit button */
		gtk_widget_set_sensitive(
				glade_xml_get_widget(xml, "transmitButton"),
				FALSE);
		g_signal_handler_disconnect(glade_xml_get_widget(xml, "transmitButton"),
					    transmitHandlerId);

		gtk_button_set_label(GTK_BUTTON(toggleButton), "Open Device");

	}

}

/* need to reregister the signal this is attached to so user_data is a valid pointer */
void
on_clearButton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkListStore *packetListStore = (GtkListStore *) user_data;
	GtkTreeIter iter;
	gboolean iterIsValid;

	/* free that data column we've been dynamically allocating */
	iterIsValid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(packetListStore), &iter);
	
	while(iterIsValid)
	{
		guint8 * data;
		gtk_tree_model_get(GTK_TREE_MODEL(packetListStore), &iter,
				   DATA_BYTES_COLUMN, &data,
				   -1);

		g_free(data);

		iterIsValid = gtk_tree_model_iter_next(GTK_TREE_MODEL(packetListStore), &iter);
	}

	/* Might be nice to do something more intelligent than
	 * throw the data out */
	gtk_list_store_clear(packetListStore);
}

/* don't let the user type things that aren't valid hex */
void
on_entry_insert_text                   (GtkEditable     *editable,
                                        gchar           *new_text,
                                        gint             new_text_length,
                                        gpointer         position,
                                        gpointer         user_data)
{
	char *temp;

	strtoul(new_text, &temp, 16);

	if(*temp == '\0')
		return;

	g_signal_stop_emission_by_name(G_OBJECT(editable), "insert_text");

}

/* display the arbitration ID in base 16 */
void
arbid_cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *renderer, 
			 GtkTreeModel *list_store, GtkTreeIter *iter,
			 gpointer data)
{
	gchar temp[9];
	guint32 arbid;

	gtk_tree_model_get(list_store, iter, ARBID_COLUMN, &arbid, -1);
	
	if(arbid != ICS_NO_ARB_ID)
	{
		g_snprintf(temp, sizeof(temp), "%x", arbid);
		g_object_set(G_OBJECT(renderer), "text", (gchararray) temp, NULL);
	}
	
}

void
data_bytes_cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *renderer, 
			 GtkTreeModel *list_store, GtkTreeIter *iter,
			 gpointer data)
{
	gchar temp[40];
	gint offset;
	gpointer data_temp;
	guint8 * data_bytes; /* Length is in the last byte because it's probably better
				than adding another member to our list store */
	gint i;

	gtk_tree_model_get(list_store, iter, DATA_BYTES_COLUMN, &data_temp, -1);
	data_bytes = (guint8 *) data_temp; /* apparently needed to get this to work */

	if(data_bytes)
	{
		offset = g_snprintf(temp, sizeof(temp), "%02x", data_bytes[0]); 
		for(i = 1; i < data_bytes[ICS_MAX_CAN_DATA]; ++i)
			offset += g_snprintf(temp + offset, 40 - offset, " %02x", data_bytes[i]);

		temp[offset] = 0;

		g_object_set(G_OBJECT(renderer), "text", (gchararray) temp, NULL);
	}
}

void
bus_cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *renderer, 
			 GtkTreeModel *list_store, GtkTreeIter *iter,
			 gpointer data)
{
	guint8 bus;
	const gchar * busStr;

	gtk_tree_model_get(list_store, iter, BUS_COLUMN, &bus, -1);

	switch (bus)
	{
		case ICS_NETID_DEVICE:	busStr = "Device";
				      	break;
		case ICS_NETID_HSCAN:	busStr = "HSCAN";
					break;
		case ICS_NETID_MSCAN:	busStr = "MSCAN";
					break;
		case ICS_NETID_SWCAN:	busStr = "SWCAN";
					break;
		case ICS_NETID_LSFTCAN:	busStr = "LSFTCAN";
					break;
		case ICS_NETID_FORDSCP: busStr = "Ford SCP";
					break;
		case ICS_NETID_J1708:	busStr = "J1708";
					break;
		case ICS_NETID_AUX:	busStr = "Aux";
					break;
		case ICS_NETID_JVPW:	busStr = "J1850/VPW";
					break;
		case ICS_NETID_ISO:	busStr = "ISO";
					break;
		case ICS_NETID_ISOPIC:	busStr = "ISOPIC";
					break;
		case ICS_NETID_MAIN51:	busStr = "Main51";
					break;
		case ICS_NETID_HOST:	busStr = "Host";
					break;
		default:	busStr = "Unknown bus";
				break;
	}

	g_object_set(G_OBJECT(renderer), "text", (gchararray) busStr, NULL);
}

static gboolean
incoming_packet_handler(GIOChannel *source, GIOCondition cond, gpointer temp)
{
	gchar * error;
	if(!device_process_packet(&error))
	{
		show_error_dialog(GTK_WINDOW(glade_xml_get_widget(xml, "mainWindow")), error);
		g_free(error);
	}


	/* keep this callback attached */
	return TRUE;
}

void
on_packetListStore_row_inserted		(GtkTreeModel	*treeModel,
					 GtkTreePath	*treePath,
					 GtkTreeIter	*treeIter,
					 gpointer	user_data)
{
	GtkScrolledWindow * messageListWindow = GTK_SCROLLED_WINDOW(glade_xml_get_widget(xml, "messageListWindow"));
	GtkAdjustment * vadjust = gtk_scrolled_window_get_vadjustment(messageListWindow);
	gdouble stepIncrement;
	
	g_object_get(G_OBJECT(vadjust), "step-increment", &stepIncrement, NULL);
	gtk_adjustment_set_value(vadjust, gtk_adjustment_get_value(vadjust) + stepIncrement * 1.5); /* 1.5 is a fudge factor */
	gtk_scrolled_window_set_vadjustment(messageListWindow, vadjust);
}

/*Use "active" property of GtkComboBox to get netid */
void
on_transmitButton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkComboBox * busBox;
	GtkEntry * arbIdEntry, *dataByteEntries[8];
	GtkCheckButton * extendedIDCheckButton = GTK_CHECK_BUTTON(glade_xml_get_widget(xml, "extendedIDCheckButton"));
	const gchar * arbIdStr;
	int i, temp;

	struct ics_can_msg msg;
	struct ics_packet p;

	busBox = GTK_COMBO_BOX(glade_xml_get_widget(xml, "busBox"));
	arbIdEntry = GTK_ENTRY(glade_xml_get_widget(xml, "arbIDEntry"));

	dataByteEntries[0] = GTK_ENTRY(glade_xml_get_widget(xml, "byte1Entry"));
	dataByteEntries[1] = GTK_ENTRY(glade_xml_get_widget(xml, "byte2Entry"));
	dataByteEntries[2] = GTK_ENTRY(glade_xml_get_widget(xml, "byte3Entry"));
	dataByteEntries[3] = GTK_ENTRY(glade_xml_get_widget(xml, "byte4Entry"));
	dataByteEntries[4] = GTK_ENTRY(glade_xml_get_widget(xml, "byte5Entry"));
	dataByteEntries[5] = GTK_ENTRY(glade_xml_get_widget(xml, "byte6Entry"));
	dataByteEntries[6] = GTK_ENTRY(glade_xml_get_widget(xml, "byte7Entry"));
	dataByteEntries[7] = GTK_ENTRY(glade_xml_get_widget(xml, "byte8Entry"));

	/* Relies upon ICS_NETID_* constants being in the same order
	   as in busBox */
	/* use temp because msg->netId is supposed to be unsigned */
	/* perhaps a wrapper call taking all the members of ics_can_msg is in order? OTOH,
	 * it should be ok to use libintrepidcs calls here, just not for device initialization */
	if((temp = gtk_combo_box_get_active(busBox) + 1) > 0 && temp <= ICS_NETID_LSFTCAN)
	{
		msg.netid = temp;
	}
	else
	{
		show_error_dialog(glade_xml_get_widget(xml, "mainWindow"), "Sorry, only CAN is currently supported.");
		return;
	}

	arbIdStr = gtk_entry_get_text(arbIdEntry);
	msg.can_id = strtoul(arbIdStr, NULL, 16);
	//g_free(arbIdStr);

	if(msg.can_id > 0x7ff || gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(extendedIDCheckButton)))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extendedIDCheckButton), TRUE);
		msg.is_extended_id = 1;
	}
	else
	{
		msg.is_extended_id = 0;
	}

	msg.is_remote_frame = 0; /* Does anyone want to do this? A checkbox could be added...*/

	/* get data bytes */
	for(i = 0; i < 8; ++i)
	{
		const gchar * text;
		if(*(text = gtk_entry_get_text(dataByteEntries[i])) == NULL)
		{
			msg.data[i] = NULL;
			break;
		}

		msg.data[i] = strtoul(text, NULL, 16);
		//g_free(text);
	}

	if(msg.data[0])
	{
		msg.data_len = i;
	}
	else
	{
		show_error_dialog(GTK_WINDOW(glade_xml_get_widget(xml, "mainWindow")), "Cannot send packet with no data!");
		return;
	}

	ics_packet_create_can_msg(&p, &msg);

	if(ics_interface_write_packet(device_iface(), &p) == 0)
	{
		gchar * err = g_strconcat("Couldn't write transmit packet: ", ics_interface_err_str(device_iface()));
		show_error_dialog(GTK_WINDOW(glade_xml_get_widget(xml, "mainWindow")), err);
		g_free(err);

		return;
	}
}

/* Signal handler for closing the main window - it's best to make sure we clean up before quitting */
gboolean
close_device_and_quit                  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	GtkWidget * mainWindow = GTK_WIDGET(widget);
	GtkToggleButton * openDeviceButton = GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "openDeviceButton"));

	if(gtk_toggle_button_get_active(openDeviceButton))
	{
		gtk_toggle_button_set_active(openDeviceButton, FALSE);
	}

	gtk_main_quit();

	return FALSE; /* only run once */
}

