#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "callbacks.h"

#define XML_FILENAME "ithinkican.glade"

GladeXML *xml;

/**
 * Create appropriate GtkCellRenderes for each column and hook up cell data functions if necessary.
 * Code to do this for columns that do not need it currently is #if 0'd.
 */
void
initialize_tree_view(GtkTreeView * treeView)
{
		GtkCellRenderer * renderer;
		GtkTreeViewColumn * col;

		/* Timestamp Column */
		renderer = gtk_cell_renderer_text_new();
		col = gtk_tree_view_column_new_with_attributes("Timestamp",
								renderer,
								"text", TIMESTAMP_COLUMN,
								NULL);
		gtk_tree_view_append_column(treeView,col);

		/* Tx Column */
		renderer = gtk_cell_renderer_toggle_new();
		col = gtk_tree_view_column_new_with_attributes("TX",
								renderer,
								"active", TX_COLUMN,
								NULL);
#if 0
		gtk_tree_view_column_set_cell_data_func(col,
							renderer,
							tx_cell_data_func,
							NULL,
							NULL);
#endif
		gtk_tree_view_append_column(treeView,col);

		/* Err Column */
		renderer = gtk_cell_renderer_toggle_new();
		col = gtk_tree_view_column_new_with_attributes("ERR",
								renderer,
								"active", ERR_COLUMN,
								NULL);
#if 0
		gtk_tree_view_column_set_cell_data_func(col,
							renderer,
							err_cell_data_func,
							NULL,
							NULL);
#endif
		gtk_tree_view_append_column(treeView,col);

		/* Arb ID Column */
		renderer = gtk_cell_renderer_text_new();
		col = gtk_tree_view_column_new_with_attributes("Arbitration ID",
								renderer,
								NULL);
		gtk_tree_view_column_set_cell_data_func(col,
							renderer,
							(GtkTreeCellDataFunc) arbid_cell_data_func,
							NULL,
							NULL);
		gtk_tree_view_append_column(treeView,col);

		/* Description Column */
		renderer = gtk_cell_renderer_text_new();
		col = gtk_tree_view_column_new_with_attributes("Description",
								renderer,
								"text", DESCRIPTION_COLUMN,
								NULL);
		gtk_tree_view_append_column(treeView,col);


		/* Data Bytes Column */
		renderer = gtk_cell_renderer_text_new();
		col = gtk_tree_view_column_new_with_attributes("Data bytes",
								renderer,
								NULL);
		gtk_tree_view_column_set_cell_data_func(col,
							renderer,
							(GtkTreeCellDataFunc) data_bytes_cell_data_func,
							999,
							NULL);
		gtk_tree_view_append_column(treeView, col);

		/* Bus Column */
		renderer = gtk_cell_renderer_text_new();
		col = gtk_tree_view_column_new_with_attributes("Bus",
								renderer,
								NULL);
		gtk_tree_view_column_set_cell_data_func(col,
							renderer,
							(GtkTreeCellDataFunc) bus_cell_data_func,
							NULL,
							NULL);
		gtk_tree_view_append_column(treeView,col);
}

int
main (int argc, char *argv[])
{
  GtkWidget *mainWindow;

  GtkWidget * messageListView;
  GtkListStore * packetListStore;
  GtkComboBox * busBox;

  gchar * xml_file_path;

  gtk_set_locale ();
  gtk_init (&argc, &argv);


  /* Open the Glade file defining the GUI */
  xml_file_path = g_strconcat(PACKAGE_DATA_DIR, "/", XML_FILENAME, NULL);
  xml = glade_xml_new(xml_file_path, NULL, NULL);
  g_free(xml_file_path);

  /* See ithinkican.glade for signal connections */
  glade_xml_signal_autoconnect(xml);

  messageListView = glade_xml_get_widget(xml, "messageListView");
  
  /* this MUST be done BEFORE setting the model!!! */
  initialize_tree_view(GTK_TREE_VIEW(messageListView));

  /* initialization and setup of packetListStore */
  packetListStore = gtk_list_store_new (  NUM_COLUMNS,
					  G_TYPE_DOUBLE, 
					  G_TYPE_BOOLEAN,
					  G_TYPE_BOOLEAN,
					  G_TYPE_UINT, /* uint 32 */
					  G_TYPE_STRING,
					  G_TYPE_POINTER, /* uint8[ICS_MAX_CAN_DATA], dynamically allocated */
					  G_TYPE_UCHAR); /* wanted G_TYPE_UINT8 but that's life */

  gtk_tree_view_set_model(GTK_TREE_VIEW(messageListView),
					GTK_TREE_MODEL(packetListStore));

  g_signal_connect((gpointer) packetListStore,
		   "row-inserted",
		   G_CALLBACK(on_packetListStore_row_inserted),
		   NULL);

  g_signal_connect((gpointer) glade_xml_get_widget(xml, "openDeviceButton"),
		   "toggled",
		   G_CALLBACK(on_openDeviceButton_toggled),
		   packetListStore);

  g_signal_connect((gpointer) glade_xml_get_widget(xml, "clearButton"),
		   "clicked",
		   G_CALLBACK(on_clearButton_clicked),
		   (gpointer) packetListStore);

  /* make sure the netID box always has a value (it starts at -1) */
  busBox = GTK_COMBO_BOX(glade_xml_get_widget(xml, "busBox")); 
  gtk_combo_box_set_active(busBox, 0);

  mainWindow = glade_xml_get_widget(xml, "mainWindow");
  gtk_widget_show (mainWindow);

  gtk_main ();
  return 0;
}

